@@ -148,8 +148,8 @@ def plot_stats(inputdir=".", outputdir=".",
         sms = SnakeMakeStats("%s/stats.txt" % inputdir, N=N)
         sms.plot_and_save(outputdir=outputdir, filename=filename)
     except Exception as err:
-        logger.error(err)
-        logger.error("Could not process %s/stats.txt file" % inputdir)
+        logger.warning(err)
+        logger.warning("Could not process %s/stats.txt file" % inputdir)
 
 
 class ModuleFinderSingleton(object):
@@ -941,8 +941,8 @@ class PipelineManagerBase(object):
         Module(self.name).check(mode)
 
         if matplotlib:
-            import matplotlib
-            matplotlib.use('Agg')
+            import matplotlib as mpl
+            mpl.use(matplotlib)
 
     def _get_snakefile(self):
         return self._snakefile
@@ -975,6 +975,28 @@ class PipelineManagerBase(object):
         cleaner.files_to_remove.extend(extra_files_to_remove)
         cleaner.add_makefile()
 
+    def get_html_summary(self, float="left", width=30):
+        import pandas as pd
+        import pkg_resources
+        vers = pkg_resources.require("sequana_{}".format(self.name))[0].version
+        df_general = pd.DataFrame({
+            "samples": len(self.samples), 
+            "paired": self.paired, 
+            "sequana_{}_version".format(self.name): vers}, 
+            index=["summary"])
+
+        from sequana.utils.datatables_js import DataTable
+        datatable = DataTable(df_general.T, 'general', index=True)
+        datatable.datatable.datatable_options = {'paging': 'false',
+                                             'bFilter': 'false',
+                                              'bInfo': 'false',
+                                               'header': 'false',
+                                              'bSort': 'true'}
+        js = datatable.create_javascript_function()
+        htmltable = datatable.create_datatable(style="width: 20%; float:left" )
+        contents = """<div style="float:{}; width:{}%">{}</div>""".format(float,
+            width, js + htmltable)
+        return contents
 
 
 class PipelineManagerGeneric(PipelineManagerBase):
@@ -1729,6 +1751,7 @@ def build_dynamic_rule(code, directory):
 
 """
 def __add_stats_summary_json(json_list, parser):
+    # used by the denovo pipeline only
     if not parser.stats:
         return
     for jfile in json_list:
